/*------------------------------------------------------------------------------------------------------------
 IR-Fernbedienung Sony RM-AMU096
  Realisiert mit ATtiny25
 
 Version 1.0
 
 
 Interner RC Oszillator mit 8MHz (Vorteiler /8 aus)
 
 
 
 Layout:
                      Reset  1    8    +5V
         L=Snooze       PB3  2     7    PB2  Rckmeldung: Sony Gert ist eingeschaltet (Bisher nicht impelmentiert)
         L=Einschalten  PB4  3     6    PB1  unbenutzter Ausgang
                        GND  4     5    PB0: IR-Led-Ausgang
 
  Wird PB4 verndert, wird der Tuner mittels IR-Pulsen eingeschaltet (PB4=L) bzw ausgeschaltet (PB4=H) 
  Wird PB3 verndert, wird die Lautstrke auf MIN gestellt (PB3=L) bzw. auf 11 (PB3=H) 
  Die IR-LED hngt an PB0 und geht ber 150R an GND.
 
*/



#define F_CPU 8000000UL	/* CPU clock in Hertz */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define U8 unsigned char
#define U16 unsigned int
#define U32 unsigned long
#define I8 signed char
#define I16 int

#define BIT0 1
#define BIT1 2
#define BIT2 4
#define BIT3 8
#define BIT4 16
#define BIT5 32
#define BIT6 64
#define BIT7 128

#define IR_POWER      0b100000010101
#define IR_AUDIO_IN   0b100000011101
#define IR_VOL_UP     0b100000010010
#define IR_VOL_DN     0b100000010011
#define IR_TUNER      0b100000100001
    
#define   DO_IR_ON  PORTB|= BIT0
#define   DO_IR_OFF PORTB&=~BIT0

U8 g_sleepAllowed1;
U8 g_sleepAllowed2;

//--------------------------------------------------------
void sendIr(U32 value, U8 bitcount){
//--------------------------------------------------------
register U32 mask=1;
register U8 puls;

//Preamble: 2400s = 96 Pulses
  for (puls=0; puls<96; puls++){
    DO_IR_ON;
    _delay_us(10);
    DO_IR_OFF;
    _delay_us(14);
  }
  _delay_us(599);

  while (bitcount--){
    if (value & mask){
      for (puls=0; puls<48; puls++){ //1200s = 48 pulses
        DO_IR_ON;
        _delay_us(10);
        DO_IR_OFF;
        _delay_us(14);
      }
    }else{
      for (puls=0; puls<24; puls++){//600s = 24 pulses
        DO_IR_ON;
        _delay_us(10);
        DO_IR_OFF;
        _delay_us(14);
      }
    }
    mask<<=1;
    _delay_us(599);
  }
}
//--------------------------------------------------------
void sendIr3(U32 value, U8 bitcount){
//--------------------------------------------------------
  sendIr(value, bitcount);
  _delay_ms(25);
  sendIr(value, bitcount);
  _delay_ms(25);
  sendIr(value, bitcount);
  _delay_ms(25);
}

//--------------------------------------------------------
void ir_powerOn(void){  //Power up and switch to quiet input
//--------------------------------------------------------
register  U8 i; 
  sendIr3(IR_POWER,12);
  for (i=0; i<5; i++){
    _delay_ms(500);
    sendIr3(IR_AUDIO_IN,12);
  }  
}

//--------------------------------------------------------
void ir_tuner(void){ //Switch to tuner
//--------------------------------------------------------
  sendIr3(IR_TUNER,12);
  _delay_ms(50);
}

//--------------------------------------------------------
void ir_volumeUp(U8 n){ //increase volume slowly by "n" 
//--------------------------------------------------------
register U8 i;
  for (i=0; i<n; i++){
    sendIr3(IR_VOL_UP,12);
    _delay_ms(650);
  }
}

//--------------------------------------------------------
void ir_volumeDown(void){
//--------------------------------------------------------
register U8 i;
  for (i=0; i<145; i++){  //5s Volume Down.  16*600s+25ms=34,6ms /Telegramm 
    sendIr(IR_VOL_DN, 12);
    _delay_ms(25);
  }
}
//--------------------------------------------------------
void ir_quiet(void){
//--------------------------------------------------------
  sendIr3(IR_AUDIO_IN,12);
  _delay_ms(150);
  ir_volumeDown();
}


//--------------------------------------------------------
void ir_powerOff(void){
//--------------------------------------------------------
  sendIr3(IR_AUDIO_IN,12);
  _delay_ms(100);
  sendIr3(IR_POWER,12);
  _delay_ms(2500);
}

ISR(PCINT0_vect){
//--------------------------------------------------------
  //Mache nix. Interrupt dient nur zum aufwachen.
}

//--------------------------------------------------------
U8 getPowerRequest(void){
//--------------------------------------------------------
//Filtert digitalen Eingang, damit nicht jeder EMV-Knacks die Anlage schaltet
static U8 filter;
static U8 state;

  g_sleepAllowed1=0;
  if (PINB & BIT4){
    if (filter < 100){
      filter++;
    }else{
      state=0; 
      g_sleepAllowed1=1;
    }
  }else{
    if (filter > 0){
      filter--;
    }else{
      state=1; 
      g_sleepAllowed1=1;
    }
  }
  return state;
}
//--------------------------------------------------------
U8 getSnoozeRequest(void){
//--------------------------------------------------------
//Filtert digitalen Eingang, damit nicht jeder EMV-Knacks die Anlage schaltet
static U8 filter;
static U8 state;

  g_sleepAllowed2=0;
  if (PINB & BIT3){
    if (filter < 100){
      filter++;
    }else{
      state=0; 
      g_sleepAllowed2=1;
    }
  }else{
    if (filter > 0){
      filter--;
    }else{
      state=1; 
      g_sleepAllowed2=1;
    }
  }
  return state;
}
//--------------------------------------------------------
int main(void){
//--------------------------------------------------------
register U8 memoPower, memoSnooze; 
U8 i;
  DDRB = BIT0|BIT1;
  PORTB= BIT2|BIT3|BIT4;  //IR-LED aus, Pullup an Eingnge
  PRR=(1<<PRTIM1)|(1<<PRTIM0)|(1<<PRUSI)|(1<<PRADC); //Strom sparen: 4,9 statt 6,5mA

  //Power-Down modus ermglichen durch Pin change Interrupts
  GIMSK = (1<<PCIE); //Pin change Interrupts enable
  PCMSK =(1<<PCINT3)|(1<<PCINT4); //sensitiv auf Pins fr Snooze und Power


  _delay_ms(800);
  for (i=0; i<200; i++){
    getPowerRequest();
    getSnoozeRequest();
  }

  sei();

  memoPower = getPowerRequest();
  memoSnooze = getSnoozeRequest();
  while(1){  
    _delay_ms(1); //Mit dieser Verzgerung sttigt der Filter in 100ms
    if (g_sleepAllowed1 && g_sleepAllowed2){ //Schlafen nur erlaubt, wenn Eingangsfilter gesttigt
      MCUCR = (1<<BODS)|(1<<SE)|(1<<SM1); //Sleep enable; Power-Down Mode
      asm("SLEEP"); //Jetzt brauche ich nur noch ca. 0.0001mA (ca. 1 mAh pro Jahr)
      //Bei nderungen an PB3 oder PB4 wacht der Prozessor wieder auf.
    } 

    if (getSnoozeRequest()!=memoSnooze){//Snooze
      if (0==(memoSnooze = getSnoozeRequest())){ 
        ir_volumeUp(11);
      }else{  
        ir_volumeDown();
      }
    }
    if (getPowerRequest()!=memoPower){//Power
      if (0==(memoPower = getPowerRequest())){ 
        ir_powerOff();
      }else{  
        ir_powerOn();
        ir_quiet();
        ir_tuner();
        ir_volumeUp(11);
      }
    }
  }//While 1
  
  return 0;
}
// EOF----------------------------------------------------------------------------------------------------------
